@@ -1,6 +1,3 @@
-#source("update.fluxes.r")
-#source("DistFlux.r")
-
 ################################################################################
 # initialisation and input checks for Dynamic TOPMODEL
 #-------------------------------------------------------------------------------
@@ -14,17 +11,8 @@ ResetFluxes<- function(flows, ichan)
   return(flows)
 }
 
-ResetStorages <- function(stores, ichan)
-{
-  # tidy up
-
-  # remove overland flows - also allocated in RouteFlow routine
-  stores[, "ex"]<- 0
-  return(stores)
-}
-
 # check that tabular object has given column names
-check.cols <- function(obj, names, stop=T)
+check.cols <- function(obj, names, stop=TRUE)
 {
   msg <- NULL
   for(nm in names)
@@ -38,7 +26,8 @@ check.cols <- function(obj, names, stop=T)
   return(msg)
 }
 
-check.and.fix.input <- function(envir)
+# checking the input for potential errors, fix them and report if specified
+check.and.fix.input <- function(envir, show.warnings=FALSE, show.errors=FALSE)
 {
 
   errs <- NULL
@@ -98,20 +87,7 @@ check.and.fix.input <- function(envir)
    # envir$weights[not.unity,] <- normalise.vector(weights[not.unity,])
   }
 
-  if(length(warns)>0){lapply(warns, warning)}
-  if(length(errs)>0)(stop(errs))
 
-#      cur.env <-  environment()
-#      # copy values to the specified environment, typicaly the  current environment in
-#      # the calling context
-#      if(!is.null(calling.env))
-#      {
-#        for(n in ls(cur.env, all.names=TRUE))
-#        {
-#          # assign name-value in this envir to that specified
-#          assign(n, get(n, cur.env), calling.env)
-#        }
-#      }
   return(envir)
 }
 
@@ -156,7 +132,7 @@ init.base.flows <- function(groups,
   # default
   Qbf0 <- r * groups$area
 #  ra[ichan] <- 0  # # recharge to river is by redistributed baseflow  only?
-  Qbf0 <- tryCatch(Qbf0 <- solve(ItW, ra), silent =T)
+  Qbf0 <- tryCatch(Qbf0 <- solve(ItW, ra), silent =TRUE)
   qbf0 <- Qbf0/groups$area
   # if matrix singular somthing is wrong with the way the matrix has been set-up
   # need to check in CheckInput
@@ -273,12 +249,12 @@ init.stores <- function(groups,
 
   # initial base flows estimated from initial recharge assumming steady state
   # see Beven 2012 equation 6.1.21
-  sdbar<- pmax(-groups$m * log(flows$qbf/q0), 0, na.rm=T)
+  sdbar<- pmax(-groups$m * log(flows$qbf/q0), 0, na.rm=TRUE)
 
   # Assign mean value to areal effective storage deficits (no -ve storages)
   if(any(sdbar<=0))
   {
-    LogEvent("Calculated initial storage deficit -ve, setting to 0 (saturation)")
+  #  LogEvent("Calculated initial storage deficit -ve, setting to 0 (saturation)")
   }
   stores$sd <- sdbar
   if(any(stores$sd>groups$sd_max))
@@ -326,16 +302,23 @@ InitialiseChannelFlows <- function(flowlengths, dt, deltaR, q0, chanV)
 
 # use the specified soil /hydraulic properties if supplied or assign from defaults
 init.hru<- function(groups,
-                            params=def.hsu.par(),
-                            chan.par=def.chan.par(),
-							ichan=1, rain=NULL)
+                    params=def.hsu.par(),
+                    chan.par=def.chan.par(),
+							      ichan=1, rain=NULL)
 {
   # groups is table of area groupings and params is list of default parameter values
   # Search for columns for saturation transmissivity ln_t0, max root zone storage srMax
   # and exponential transmissivity profile factor m
+  # initialise missing parameters with defaults
+  params <- def.hsu.par()
+  nms <- setdiff(names(params), colnames(groups))
+  params<- params[nms]
+  groups <- apply_params(groups, params)
 
   # add missing params using defaults. groups' parameters override defaults
-	nms <- setdiff(colnames(groups), names(params))
+#	nms <- setdiff(colnames(groups), names(params))
+
+
   #groups[igroup,] <- as.vector(merge.lists(params, def.hsu.par()))
 #    }
 	if(length(ichan)>0)
@@ -352,37 +335,19 @@ init.hru<- function(groups,
 	  groups[ichan,]$td <- chan.par$td  # all drainage will be routed directly to base flow
 	}
 	# checj taht groups wetness is in decreasing order
-	#if(!all(order(groups[-ichan,$atb.bar, decreasing=T)==groups[-ichan,]$order)){warning("Wetness index not in decreasing order of HRU number")}
+	#if(!all(order(groups[-ichan,$atb.bar, decreasing=TRUE)==groups[-ichan,]$order)){warning("Wetness index not in decreasing order of HRU number")}
   # labels for groups default to ids (shoudl these be sequential?)
   if(is.null(groups$tag)){groups$tag<-groups$id}
-	# max base flow when deficit =0
-	try(groups$qbmax <- exp(groups$ln_t0-groups$atb.bar), silent=T)
+
 	n.gauge <- ifelse(is.null(rain),
 										1,
 										ncol(rain))
   # ensure gauge reading supplied is withing range
   groups$gauge.id <- pmin(groups$gauge.id, n.gauge)
 
-  # any drainage params
-  groups <- init.drainage.params(groups)
   return(groups)
 }
 
-# add column for parameter unless it already exists
-AddParam <- function(groups, name, val)
-{
-  if(!name %in% names(groups))
-  {
-    # add column with default values if not found
-    groups <- cbind(groups, list(val))
-    names(groups)[length(groups)]<-name
-    return(groups)
-  }
-
-  # just return the input
-  return(groups)
-
-}
 
 # returns the input ts subsetted by a time range
 # either bound or neither can be supplied, if a bound is not supplied then the corresponding bound of the
@@ -394,7 +359,7 @@ GetTimeSeriesInputRange <- function(series, start, endt,
                                     dt=1, verbose=TRUE)
 {
   if(is.null(series)){return(series)}
-  #if(length(series)==0){browser()}
+
 	# subset of columns if specified
 	series <- series[,cols]
 	res <- series[index(series)>=start & index(series)<=endt]
@@ -431,35 +396,65 @@ GetTimeSeriesInputRange <- function(series, start, endt,
 require(xts)
 
 # if the max value of the given series or value is greater than 1m it implies that the values are in mm/hr.
-# ensure consistency by dividing by the given factor.
-convert.values <- function(x, msg="Value(s) converted to m/hr", max=1, fact=1000)
+# ensure consistency by dividing by the given factor. Would have been better do supply inputs in mm/hr. Too late now!
+# also replace any NAs with zeroes
+convert.values <- function(x,
+                           msg="Value(s) converted to m/hr", max=1, fact=1000)
 {
-  if(max(x, na.rm = T)> max)
+  # convert any NAs to zeroes
+  ina <- which(is.na(x[]))
+  if(length(ina)>0)
   {
-    message(msg)
+    x[ina] <- 0
+  }
+  if(length(x) > 0 && max(x, na.rm = TRUE) > max)
+  {
+    #message(msg)
     x <- x/fact
   }
   return(x)
 }
 
+# configure any flows external to the channel
+init_upstream_inputs <- function(upstream_inputs, groups, dt)
+{
+  upstream_inputs <- lapply(upstream_inputs,
+    function(upstream_input)
+    {
+      # number of time steps before the flow reaches the outlet (dt in hours)
+      idelay <- round(upstream_input$flow_dist_to_outlet/max(groups$vchan, na.rm=TRUE)/dt)
+
+      nobs <- nrow(upstream_input$flow)
+      iend <- nobs - idelay
+      # don't worry too much about what happens in the first few time steps; replicate the first input
+      # (or could make zero)
+      upstream_input$qshift <- rbind(upstream_input$flow[rep(1,idelay),],
+                      upstream_input$flow[1:iend,])
+
+      return(upstream_input)
+
+    }
+  )
+  return(upstream_inputs)
+}
+
 init.input <- function(groups, dt, ntt,
                       weights, rain, pe, routing=NULL,
 					            ichan=1, i.out=1,
                       qobs=NULL,   # specific or absolute discharge
                       qt0=1e-4,    # initial discharge in m/hr
                       dqds=NULL,
-                      disp.par, run.par,
+                      disp.par,
                       sim.start, sim.end,
                       calling.env = NULL)
 {
   if(is.null(i.out)){i.out<-1}
-  # combine lists, supplied attributes overriding the defaults and missing values being supplied
-  # by defaults
-  disp.par <- merge.lists(disp.par(), disp.par)
-  run.par <- merge.lists(def.run.par(), run.par)
+
+  # combine lists, supplied attributes overriding the defaults and missing values being supplied by defaults
+  disp.par <- merge.lists(get.disp.par(), disp.par)
+
     # need this for $ notation
   groups<-data.frame(groups)
-  runid <- run.par$id
   qmax <- disp.par$max.q
 
   if(length(rain)==0)
@@ -468,18 +463,24 @@ init.input <- function(groups, dt, ntt,
   }
 
   tz <- ""
-  # use the rainfall data to infer the time step and run start and end times, if not otherwise specified
+  # use the rainfall data to infer the time step and run start and end times,
+  # if not otherwise specified
   if(is.null(dt))
   {
     dt <- as.numeric(difftime(index(rain)[2], index(rain)[1], units="hour"))
   }
 
-  # default limits of run period inferred from time series input
-  sim.start <- max(start(rain), as.POSIXct(sim.start), na.rm=T)
-  sim.end <- min(end(rain), as.POSIXct(sim.end), na.rm=T)
+  # ensure using POSXIct - fails
+  index(rain) <- as.POSIXct(index(rain))
 
   dispStart <- sim.start + disp.par$"graphics.delay"*3600 # hours -> seconds
 
+  # note that DTM uses an input time step of hours
+  # everything is based around the rainfall time series
+  tms <- index(rain)
+  sim.start <- tms[1]
+  sim.end <- tms[length(tms)]
+
   # check that the river reach distribution is of the correct dimensions -
   # should be ngroup x nreach, where nth row determines the distribution of the
   # nth group into the river network
@@ -496,57 +497,54 @@ init.input <- function(groups, dt, ntt,
 #  ichan <- 1:nchan
   weights <- as.matrix(weights)
 
-  # logging output (defaults to console if not specified)
-  if(nchar(run.par$log.out)>0)
-  {
- #   glogout <- file(run.par$log.out)
-    # route messages to a file, otherwise sent straight to console
-   # sink(run.par$log.out, type="message", append=T)
-  }
-
-  # check consistent matrix dims, array sizes and values
-  # check times series input in correct format and duration
-
-  LogEvent("Dynamic TOPMODEL run started")
-
   # times for output series, dt in hours
-  times <- seq(sim.start, sim.end, by = 3600*dt)
-  nmax <- length(times)
+  nmax <- length(tms)
 
   # create dummy pe and qobs if none supplied, Q) can be used to set absolute discharge max limit
-  if(is.null(pe)){
-    pe <- xts(rep(0,nmax), order.by=times)
+  if(is.null(pe))
+  {
+  	pe <- rain
+  	pe[] <-0
   }
-  # locate and fix any NA records
-  pe[which(is.na(pe))] <-0
 
   # ensure everything in same units
   rain <- convert.values(rain)
   qt0 <- convert.values(qt0)
-
   pe <- convert.values(pe)
 
-  qobs <- GetTimeSeriesInputRange(qobs, sim.start, sim.end, verbose=FALSE)
+  #qobs <- GetTimeSeriesInputRange(qobs, sim.start, sim.end, verbose=FALSE)
 
   # dummy series if nothing supplied or outside range of simulation
   if(length(qobs)>0)
   {
 		qobs <- xts(qobs)
 
+		index(qobs) <- as.POSIXct(index(qobs))
 		# want a specific input, observed values often (always?) in cubic m^3/s
-		qobs <- convert.values(qobs, msg="Observed flows to m/hr: check that specfic discharge in m/hr has been supplied")
+		qobs <- convert.values(qobs, msg="Observed flows converted to m/hr: check that specfic discharge in m/hr has been supplied")
+
+		# check (qobs should be in m/hr)
+    qin <- sum(rain, na.rm = TRUE)
+    qout <- sum(qobs[,1], na.rm=TRUE)
+
+    if((qout-qin)/qout> 0.25)
+    {
+      #warning("Water balance out by >25%, check rainfall and /or observations")
+    }
+		if(length(qt0)==0)
+		{
+			# if the initial value not suppplied then take as first observation
+			qt0 <- as.numeric(qobs[1, 1])
+			message(paste0("Initialising fluxes using first discharge observation of ", round(qt0*1000, 1), " mm/hr"))
+		}
   }
 
-  # use the overland velocity as proxy for channel routing velocity
- # def.chan.par$vchan <- vchan
   # initialise storages and deficits for each group add static parameters to
   # areal groups if a soils map provided then fix up the soil parameters from
   # here, otherwise continue to use default
   groups <- init.hru(groups, ichan=ichan, rain=rain)
-  # Input rainfall, observed flows and p.e.
-  rain <- GetTimeSeriesInputRange(rain, sim.start, sim.end, verbose=FALSE)
-  pe <- GetTimeSeriesInputRange(pe, sim.start, sim.end, verbose=FALSE)
-  # add the actual evap
+
+    # add the actual evap
   evap <- cbind("pe"=pe, "ae"=NA)
 
   try(check.and.fix.input(environment()))
@@ -558,19 +556,19 @@ init.input <- function(groups, dt, ntt,
 											 ichan,	dt)
   # max number of time steps that river flux can be shifted
   # Time series of total output flows from output reach(s). add enough extra elements to hold time delayed flows
-  n.shift <- nrow(routing)
-  times.ex <-seq(sim.start, sim.end+nrow(routing)*dt*3600, by = 3600*dt)
+  n.shift <- nrow(routing) + 100
+  times.ex <-seq(sim.start, sim.end+n.shift*dt*3600, by = 3600*dt)
 
   nms <- groups[ichan,]$id
 
   # numbet of outlet reaches to produce flow output
   nout <- ncol(routing)
-  Qr <- xts(matrix(0,nrow=nmax+nrow(routing), ncol=nout), order.by=times.ex)
 
-  #colnames(Qr)<- nms
+  # create the output series with enough entries to account for extra flow routed
+  Qr <- xts(matrix(0,nrow=n.shift+nmax, ncol=nout), order.by=times.ex)
+
 # total overland flow contribution
-  Qof <- xts(matrix(0,nrow=nmax,ncol=1), order.by=times)
-
+  qof <- xts(matrix(0,nrow=nmax,ncol=1), order.by=tms)
 
   #*****************************************************************************
   # initialisation of flows, storages and deficits for each areal group flows
@@ -586,14 +584,17 @@ init.input <- function(groups, dt, ntt,
   # typical values for ln_t0 are in range (-7,8).
   # Could examine catchment and its observed discharges to calibrate this
   groups$qbf_max <- exp(groups$ln_t0-groups$atb.bar)
+  
+  # using the slope - correct! 
+  if(!is.null(groups$sbar))
+  {
+    # limiting transmissivty x mean slope
+    groups$qbf_max <- exp(groups$ln_t0)*groups$sbar
+  }
+  
   # huge value ensures no excess in river (or set to zero to ensure that ay base flow in river produces "surface" run-off)
   groups$qbf_max[ichan]<-1e6
 
- # weights <- spam::as.spam(weights)
-
-  # in the summer, effective rainfall recharge will be affected significantly be ea.
-  # as we don't know the storages yet, apply a
-
   # estimate for initial base fluxes given a steady state
   flows <- init.fluxes(groups=groups,
                       weights,           # flux distribution matrix
@@ -605,15 +606,13 @@ init.input <- function(groups, dt, ntt,
   stores <- init.stores(groups, flows, ichan,
                          qt0)
 
-
   # initial determined from storages vs. deficit
 
   # index in time series after which to start showing results
-  i.start <- which(times>=sim.start)[1]
+  i.start <- which(tms >= sim.start)[1]
+
   if(is.na(i.start))
   {stop(paste("Error determining start of simulation period ", dispStart))}
-  # 	LogEvent(paste("initialising time series: start ", as.character(start)))
-
 
   # structure to save the average storage deficit. ae, river input, water balance etc etc
   dat <- data.frame("ae"=rep(0,nmax))
@@ -621,14 +620,16 @@ init.input <- function(groups, dt, ntt,
                     #"qr"=rep(0,nmax),
                     #"qriv.in"=rep(0,nmax),   # riveer input fluxes
                     #"ae"=rep(0,nmax))
-  data.out <- xts(dat, order.by=times)
+  data.out <- xts(dat, order.by=tms)
 
-  qriver <- xts(matrix(rep(0,nmax,each=nchan), ncol=nchan), order.by=times)
+  qriver <- xts(matrix(rep(0,nmax,each=nchan), ncol=nchan), order.by=tms)
   hsus <- groups[-ichan,]
   nhsu <- nrow(hsus)
 
-  fluxes <- array(dim=c(nmax, ngroups, 7), dimnames=c( "it", "hsu", "flux"))
-  storages <- array(dim=c(nmax, ngroups, 4), dimnames=c( "it", "hsu", "store"))
+  # array to hold state of response unit fluxes through time
+  fluxes <- array(dim=c(nmax, ngroups, 7)) #, dimnames=c( "it", "hsu", "flux"))
+  # array that holds state of response unit storages through simulation
+  storages <- array(dim=c(nmax, ngroups, 4)) #, dimnames=c( "it", "hsu", "store"))
   # graphics interval expressed in hours: convert to time steps
 
   # start of outer time stepping logic using a time step at each rainfall observation
@@ -638,29 +639,21 @@ init.input <- function(groups, dt, ntt,
   # initialise excess (overland) flow to 0
   q.ex <- rep(0, nrow(groups))
 
-# initialisation
-    # run this number of time steps to get an equilibrium with
-  # rainfall and specified initial discharge
-  # number of time steps to wait before start
-#   n.step <- round(run.par$sim.delay/dt)
-#
-#   flows <- init$flows
-#   stores <- init$stores
   # set  first discharge estimates to value specified (total, not specific)
   # extend into lag period whilst flux is stil in transit. If this isn't done then it appears that
   # there is a period when no flux enters and we see a initial dip in the hydrograph
- # Qr[it:(it+n.shift),]<- qt0*sum(groups$area)
   nroute <- length(routing)
   flows1 <- flows
-  # at steady state the flows into the channel equal those at the outlet
+  # steady state - (total) flow into the channel equal those at the outlet
   flows1$qin[ichan] <- qt0*sum(groups$area)
 
   Qr1 <- Qr
-  # distribute the initial flow to future flows
+  # distribute initial flow to future flows so that each time step has same
   for(it1 in 1:nroute)
   {
-    Qr1 <- route.channel.flows(groups, flows1, delays=routing,
-                          weights, Qr1, it=it1, dt, ichan)
+    Qr1 <- route.channel.flows(groups, flows1, stores=stores,
+    													 delays=routing,
+                                weights, Qr1, it=it1, dt, ichan)
   }
   Qr[1:nroute,]<- qt0*sum(groups$area)
 
@@ -669,32 +662,34 @@ init.input <- function(groups, dt, ntt,
 
 # set channel deficts to a typical value
 	stores[ichan,]$sd <- groups[ichan, ]$sd_max/2
-  run.par$sim.delay <- 0
+
 
 	# initialise surface excess storage to zero
 	surf.ex <- stores$ex
 	pe.dist <- xts(order.by=index(pe), matrix(rep(pe, ngroups), ncol=ngroups))
 
   # open graphics windows, calculate interval etc
-  disp.par <-setup.output(disp.par, run.par, rain=rain, qobs=qobs)
+  disp.par <-setup.output(disp.par, rain=rain, qobs=qobs)
 
-
-    # function specifying gradient of q - s function defaults to exponential
-    if(is.null(dqds))
-    {
-        dqds <-
-        function(q, S, params)
-        {                   #
-            return(-q/params$m)
-        }
-    }
+  # function specifying gradient of q - s function defaults to exponential
+  if(is.null(dqds))
+  {
+      dqds <-
+      function(q, S, params)
+      {                   #
+          return(-q/params$m)
+      }
+  }
 
   rain <- as.xts(rain)
   pe <- as.xts(pe)
-  # ensure that if it's raining there is nil evapotranspiration
-  tm.rain <- index(rain)[which(rain>0)]
-  pe[tm.rain]<- 0
+#  pe[which(rain>0)]<-0
 
+  # ensure that if it's raining there is nil evapotranspiration
+#  tm.rain <- index(rain)[which(rain>0)]
+#  pe[tm.rain]<- 0
+
+  # trim the simulation time to the time series
   sim.end <- as.POSIXct(min(max(index(pe)), max(index(rain)), sim.end))
   cur.env <-  environment()
   # copy values to the specified environment, typicaly the  current environment in
@@ -708,23 +703,9 @@ init.input <- function(groups, dt, ntt,
     }
   }
 
-
+  # return as environment that can be loaed into the main routine
   return(cur.env)
 }
 
 
-init.drainage.params <- function(groups)
-{
-  idrained <- which(!is.null(groups$drained) && groups$drained)
-  if(length(idrained)>0)
-  {
-    # set the drained groups' parameters from the field capacity phi, drained thickness dz.drain and top of drained layer z.drain
-    # drainage depth and field capacity
-    groups[idrained,]$sd_drain_max <- groups[idrained,]$z_drain * groups[idrained,]$phi
-    groups[idrained,]$sd_drain_min <- (groups[idrained,]$z_drain-groups[idrained,]$dz_drain)* groups[idrained,]$phi
-  }
-  return(groups)
-}
-
-
 
